library rx.operators.debounce;

import 'package:rxdart/src/observable/stream.dart';

class DebounceObservable<T> extends StreamObservable<T> with ControllerMixin<T> {

  Timer _timer;
  final Duration _duration;
  bool _closeAfterNextEvent = false;

  T _lastValue;
  bool _hasEvent = false;

  DebounceObservable(Stream<T> stream, Duration duration) : _duration = duration {
    StreamSubscription<T> subscription;

    _timer = new Timer.periodic(_duration, (_) {
      if (!_hasEvent && _lastValue != null) {
        controller.add(_lastValue);

        if (_closeAfterNextEvent) controller.close();

        _timer.cancel();
      } else {
        _hasEvent = false;
      }
    });

    controller = new StreamController<T>(sync: true,
        onListen: () {
          subscription = stream.listen((T value) {
            _lastValue = value;

            _hasEvent = true;
          },
          onError: (e, s) => throwError(e, s),
          onDone: () {
            _closeAfterNextEvent = true;
          });
        },
        onCancel: () => subscription.cancel());

    setStream(stream.isBroadcast ? controller.stream.asBroadcastStream() : controller.stream);
  }

}